@@ -6,18 +6,23 @@ use warnings FATAL => 'all';
 use Carp;
 use POSIX;
 
+use MongoDB::OID;
 use List::MoreUtils qw(firstval);
 use Tie::IxHash;
 use boolean;
 
-use Sport::Analytics::NHL::Config;
-use Sport::Analytics::NHL::LocalConfig;
-use Sport::Analytics::NHL::Util;
+use Sport::Analytics::NHL::Config qw(:basic $EMPTY_NET_ID %TEAMS %SEASONS);
+use Sport::Analytics::NHL::Vars qw(:globals :mongo :basic);
+use Sport::Analytics::NHL::Util qw(:debug :format :utils);
+
+use parent 'Exporter';
+
+our @EXPORT = qw(ensure_index insert update);
 
 use Data::Dumper;
 
 use if ! $ENV{HOCKEYDB_NODB} && $MONGO_DB, 'MongoDB';
-use if ! $ENV{HOCKEYDB_NODB} && $MONGO_DB, 'MongoDB::OID';
+use if ! $ENV{HOCKEYDB_NODB} && $MONGO_DB, 'BSON::OID';
 use if ! $ENV{HOCKEYDB_NODB} && $MONGO_DB, 'MongoDB::MongoClient';
 
 =head1 NAME
@@ -162,7 +167,13 @@ A wrapper over $self->{dbh}->get_collection();
 
 =item C<remove_game>
 
+Removes a game and references to it from the Mongo DB
 
+ Arguments: the game
+
+ Returns: void
+
+ Caveat: removal of all traces is not guaranteed.
 
 =item C<set_injury_history>
 
@@ -210,6 +221,123 @@ During removal of game data, wipes a game from player's history.
 
  Returns: void
 
+=item C<bootstrap_divisions_conferences>
+
+Bootstraps the NHL divisions and conferences from the Sport::Analytics::NHL::Config definitions
+
+ Arguments: force overwrite flag (optional)
+
+ Returns: void
+
+=item C<bootstrap_teams>
+
+Bootstraps the NHL teams from the Sport::Analytics::NHL::Config definitions
+
+ Arguments: force overwrite flag (optional)
+
+ Returns: void
+
+=item C<ensure_shift_indices>
+
+Ensures the custom indices for the shifts collection.
+
+ Arguments: none
+
+ Returns: void
+
+=item C<find_items_with_games>
+
+Returns the iterator of items by given game ids. Mostly used by the website, but should get the usage expanded.
+
+ Arguments: the hashref of:
+  collection of the items
+  condition to apply
+  [optional] id_field of the collection
+  [optional] game_ids to look up
+  [optional] return_fields to return
+  [optional] count whether to return the iterator or the count
+
+ Returns: the iterator or the count of matching documents.
+
+=item C<find_player>
+
+Finds a player by given name, team, season, stage
+
+ Arguments: the player name
+            the team
+            the season
+            the stage
+
+ Returns: player's ID if the player is unambiguously found
+          dies if more than one players are found
+          warns if no players are found
+
+=item C<get_catalog_condition>
+
+Get a catalog item's condition for a search
+
+ Arguments: the catalog's name
+            the vocabulary entry name
+
+ Returns: hashref catalog_name => catalog_item_id
+
+=item C<get_item_collection>
+
+Gets a collection for a given item by name. Expect to be deprecated.
+
+ Arguments: the collection itself, or the collection name (optional)
+
+ Returns: the collection or by default the games collection.
+
+=item C<get_item_condition>
+
+Expands an item condition from vocabulary names to catalog IDs
+
+ Arguments: the condition
+
+ Returns the expanded condition or
+         the original condition
+          if the 'verbatim' keyword was specified.
+
+=item C<insert>
+
+A generic wrapper over the MongoDB::Collection's insert_one / insert_many / replace_one methods. Generates the 'inserted' / 'updated' field as well.
+
+ Arguments: the collection
+            the array of items
+
+ Returns: void
+
+=item C<prepare_event>
+
+Cleans up an event before inserting it into the Mongo DB.
+
+ Arguments: the event
+            the events hashref for tracking and debugging
+
+ Returns: void
+
+=item C<update>
+
+A generic wrapper over the MongoDB::Collection's update_one / update_many methods. Generates the 'updated' field as well.
+
+ Arguments: the one_many flag: 1 means many,
+            the collection
+            the filter to update
+            the update arguments
+
+ Returns: void
+
+=item C<insert_injuries>
+
+Inserts the data structure with injuries as crawled by Sport::Analytics::NHL::Scraper::crawl_injured_players() (q.v.),
+The players are resolved to the player ids, and the begin and the end of the injuries are marked. If the player is
+already in an injury status with the same team, the injury "end" property is extended.
+
+ Arguments: the injuries structure
+
+ Returns: void
+
 =back
 
 =cut
@@ -226,22 +354,28 @@ our %EVENT_CATALOGS = (
         penalty   => 'penalties',
         strength  => 'strengths',
 );
+our %STATS_FORCE_LC = qw(number 1 start 1 position 1 status 1 _id 1 team 1 name 1);
+our %STATS_FORCE_NUMBER = qw(last_updated 1 _id 1 attendance 1 month 1 date 1 ot 1 start_ts 1 stop_ts 1 stage 1 season 1 season_id 1);
 
 sub new ($;$) {
 
 	my $class    = shift;
 	my $database = shift || $ENV{HOCKEYDB_DBNAME} || $MONGO_DB || $DEFAULT_DB;
 
+	my $slef;
+
 	my $self = {};
 	my $host = $MONGO_HOST || $DEFAULT_HOST;
 	my $port = $MONGO_PORT || $DEFAULT_PORT;
 	$self->{client} = MongoDB::MongoClient->new(
 		host => sprintf(
 			"mongodb://%s:%d", $host, $port
-		)
+			#"mongodb://%%2Ftmp%%2Fmongodb-27017.sock"
+		), connect_timeout_ms => 60000,
 	);
 	my $db = $database || $DEFAULT_DB;
 	$self->{dbh} = $self->{client}->get_database($db);
+	$DB = $self;
 	$ENV{HOCKEYDB_DBNAME} = $db;
 	verbose "Using Mongo database $db";
 	$self->{dbname} = $db;
@@ -249,7 +383,50 @@ sub new ($;$) {
 	$self;
 }
 
-sub get_collection ($$) { shift->{dbh}->get_collection(shift) }
+sub get_collection ($$) {
+
+	my $self = shift;
+	my $collection = shift;
+
+	$CACHES->{collections}{$collection} ||= $self->{dbh}->get_collection($collection);
+	$CACHES->{collections}{$collection};
+}
+
+sub insert ($;@) {
+
+	my $collection = shift;
+	my @items      = @_;
+
+	my $name = ref $collection ? $collection->{name} : $collection;
+	debug "Inserting " . scalar(@items) . " items into $name";
+	$collection   = $DB->get_collection($collection) unless ref $collection;
+	my @insert_items = ();
+	for my $item (@items) {
+		if ($item->{_id} && $item->{inserted}) {
+			$item->{updated} = time;
+			$collection->replace_one({ _id => $item->{_id} }, $item);
+		}
+		else {
+			$item->{inserted} = time;
+			push(@insert_items, $item);
+		}
+	}
+	$collection->insert_many(\@insert_items);
+}
+
+sub update ($$$$) {
+
+	my $one_many   = shift;
+	my $collection = shift;
+	my $filter     = shift;
+	my $arg        = shift;
+
+	$collection   = $DB->get_collection($collection) unless ref $collection;
+	my $method = $one_many ? 'update_many' : 'update_one';
+	$arg->{'$set'} ||= {};
+	$arg->{'$set'}{updated} = time;
+	$collection->$method($filter, $arg);
+}
 
 sub ensure_index ($$;$) {
 
@@ -257,8 +434,8 @@ sub ensure_index ($$;$) {
 	my $index_map  = shift;
 	my $reapply    = shift || 0;
 
-	return if ! $reapply && $collection->count();
-
+	return if ! $reapply && $collection->count_documents({});
+	debug "Ensuring indices for $collection";
 	my $indices   = $collection->indexes();
 	my $method    = ref $index_map eq 'ARRAY'
 		? 'create_many'
@@ -308,7 +485,7 @@ sub insert_schedule ($@) {
 		else { 0 }
 	} map(ref $_ && ref $_ eq 'ARRAY' ? @{$_} : $_, @games);
 	$schedule_c->delete_many({_id => { '$in' => [ map {$_->{_id}} @games ] } });
-	$schedule_c->insert_many([@games]);
+	insert($schedule_c, @games);
 	debug "Inserted " . scalar(@games) . " games for season $games[0]->{season}";
 	scalar @games;
 }
@@ -320,14 +497,16 @@ sub get_existing_game_ids ($;$) {
 		stop_season => $CURRENT_SEASON, start_season => $CURRENT_SEASON
 	};
 
-	my @games = $self->get_collection('games')->find({
-		season => {
-			'$gte' => $opts->{start_season}+0,
-			'$lte' => $opts->{stop_season} +0
+	my $game_ids = [];
+	my $games_i = $self->get_collection('games')->aggregate([
+		{
+			'$project' => { _id => 1 },
 		},
-	}, {_id => 1})->all();
-
-	[ map($_->{_id}+0,@games) ];
+	]);
+	while (my $game = $games_i->next()) {
+		push(@{$game_ids}, $game->{_id});
+	}
+	$game_ids;
 }
 
 sub create_location ($$) {
@@ -343,14 +522,15 @@ sub create_location ($$) {
 	my $location_db = $locations_c->find_one({name => $location->{name}});
 	if ($location_db) {
 		if ($location_db->{capacity} < $location->{capacity}) {
-			$locations_c->update_one(
+			update(
+				0, $locations_c,
 				{ name => $location->{name} },
 				{ '$set' => { capacity => $location->{capacity} }}
 			);
 		}
 	}
 	else {
-		$locations_c->insert_one($location);
+		insert($locations_c, $location);
 	}
 	$locations_c->find_one({name => $location->{name}});
 }
@@ -365,7 +545,7 @@ sub add_new_player ($$) {
 	}
 	$player->{games}  ||= [];
 	$player->{starts} ||= [];
-	$players_c->insert_one($player);
+	insert($players_c, $player);
 	$players_c->find_one({_id => $player->{_id}});
 }
 
@@ -492,15 +672,14 @@ sub add_game_player ($$$$;$) {
 	push(@{$player_db->{games}},  $game->{_id} + 0);
 	push(@{$player_db->{starts}}, $game->{_id} + 0)
 		if defined $player_game->{start} && $player_game->{start} == 1;
+	$player_db->{career} = $player->{career};
 	$player_game->{status} ||= 'X';
 	$player_game->{start}  ||= 2;
 	set_player_statuses($player_db, $player_game, $game, $team->{name});
 	set_player_teams($player_db, $game, $team->{name});
 	set_injury_history($player_db, $game, 'OK');
-	$players_c->replace_one(
-		{ _id    => delete $player_db->{_id} },
-		$player_db,
-	)
+	$player_db->{active} //= 1;
+	insert($players_c, $player_db);
 }
 
 sub add_new_coach ($$$) {
@@ -509,7 +688,7 @@ sub add_new_coach ($$$) {
 	my $game      = shift;
 	my $team      = shift;
 
-	$coaches_c->insert_one({
+	insert($coaches_c, {
 		name  => $team->{coach},
 		teams => [{
 			start => $game->{start_ts},
@@ -559,10 +738,7 @@ sub add_game_coaches ($$) {
 				if $coach_db->{end} < $game->{start_ts};
 			$coach_db->{team}  = $team->{name};
 		}
-		$coaches_c->replace_one(
-			{ _id => $coach_db->{_id} },
-			$coach_db
-		);
+		insert($coaches_c, $coach_db);
 	}
 }
 
@@ -592,24 +768,25 @@ sub ensure_event_indices ($$$) {
 	my $event   = shift;
 	my $event_c = shift;
 
-	if ($event->{type} eq 'STOP') {
-		$event->{stopreasons} = [ map (
-			$self->get_catalog_entry('stopreasons', $_)->{_id},
-			@{$event->{stopreason}},
-		)];
-		delete $event->{stopreason};
-	}
-	my $keys = {
-		keys => [ game_id => 1 ],
-	};
-	$keys->{options} = { unique => 1 }
+	my $keys = [
+		{ keys => [ game_id => 1 ], },
+		{ keys => [ season  => 1 ], },
+		{ keys => [ stage   => 1 ], },
+		$event->{team1}   ? { keys => [ team1 => 1 ], }   : (),
+		$event->{team2}   ? { keys => [ team2 => 1 ], }   : (),
+		$event->{player1} ? { keys => [ player1 => 1 ], } : (),
+		$event->{player2} ? { keys => [ player2 => 1 ], } : (),
+	];
+	$keys->[0]{options} = { unique => 1 }
 		if $event->{type} eq 'GEND'
 		|| $event->{type} eq 'PSTR'
 		|| $event->{type} eq 'PEND';
-	push(@{$keys->{keys}}, period => 1)
+	push(@{$keys->[0]{keys}}, period => 1)
 		if $event->{type} eq 'PEND'
 		|| $event->{type} eq 'PSTR';
-	ensure_index($event_c, [ $keys ]);
+	my @indices = $event_c->indexes->list->all();
+	debug "Ensure: " . scalar(@indices) . ' vs ' . scalar(@{$keys});
+	ensure_index($event_c, $keys) unless @indices == @{$keys} + 1;
 	if (exists $event->{coordinates}{x}) {
 		$event->{coordinates}{x} += 0;
 		$event->{coordinates}{y} += 0;
@@ -617,10 +794,25 @@ sub ensure_event_indices ($$$) {
 	$event->{_id} += 0;
 }
 
-sub create_event ($$) {
+sub ensure_shift_indices () {
+
+	my $shifts_c = $DB->get_collection('shifts');
+	my @indices = $shifts_c->indexes->list->all();
+	my $keys = [
+		{ keys => [ game_id => 1 ], },
+		{ keys => [ season  => 1 ], },
+		{ keys => [ stage   => 1 ], },
+		{ keys => [ player  => 1 ], },
+		{ keys => [ team    => 1 ], },
+	];
+	ensure_index($shifts_c, $keys) unless @indices == @{$keys} + 1;
+}
+
+sub prepare_event ($$$) {
 
 	my $self       = shift;
 	my $event      = shift;
+	my $_events    = shift;
 
 	my $event_c  = $self->get_collection($event->{type});
 	my $event_db = $event_c->find_one({_id => $event->{_id}+0});
@@ -630,19 +822,32 @@ sub create_event ($$) {
 			$EVENT_CATALOGS{$field}, $event->{$field}
 		)->{_id} if exists $event->{$field};
 	}
+	numerify_structure($event);
+	if ($event->{type} eq 'STOP') {
+		$event->{stopreasons} = [ map (
+			$self->get_catalog_entry('stopreasons', $_)->{_id},
+			@{$event->{stopreason}},
+		)];
+		delete $event->{stopreason};
+	}
+	debug "ensure indices";
 	$self->ensure_event_indices($event, $event_c);
+	debug "indices ensured";
 	my $events_c = $self->get_collection('events');
 	ensure_index($events_c, [
 		{ keys => [ event_id => 1 ], options => {unique => 1} },
 		{ keys => [ game_id  => 1 ], },
 	]);
-	$events_c->insert_one({
-		type     => $event->{type},
-		event_id => $event->{_id}     + 0,
-		game_id  => $event->{game_id} + 0,
-	});
-	$event_c->insert_one($event);
-	$event->{_id};
+	push(
+		@{$_events->{events}}, {
+			type     => $event->{type},
+			event_id => $event->{_id}     + 0,
+			game_id  => $event->{game_id} + 0,
+		}
+	);
+	$_events->{types}{$event->{type}} ||= [];
+	push(@{$_events->{types}{$event->{type}}}, $event);
+	push(@{$_events->{_ids}}, $event->{_id});
 }
 
 sub remove_game ($$) {
@@ -667,15 +872,15 @@ sub remove_game ($$) {
 	for my $t (0,1) {
 		my $team = $game->{teams}[$t];
 		my $coach = $coaches_c->find_one({_id => $team->{coach}});
-		if ($coach->{name} ne 'UNKNOWN COACH') {
+		if ($coach->{name} && $coach->{name} ne 'UNKNOWN COACH') {
 			debug "Cleaning coach";
 			$coach->{games} =
 				[ grep { $_ != $game->{_id} } @{$coach->{games}} ];
-			$coaches_c->update_one({
-				_id => $coach->{_id},
-			}, {
-				'$set' => { games => $coach->{games} }
-			});
+			update(
+				0, $coaches_c, {	_id => $coach->{_id}, }, {
+					'$set' => { games => $coach->{games} }
+				}
+			);
 		}
 		for my $player (@{$team->{roster}}) {
 			debug "Cleaning player";
@@ -683,7 +888,7 @@ sub remove_game ($$) {
 				[ grep { $_ != $game->{_id} } @{$player->{games}} ];
 			$player->{starts} =
 				[ grep { $_ != $game->{_id} } @{$player->{starts}} ];
-			$players_c->update_one({
+			update(0, $players_c, {
 				_id => $player->{_id},
 			}, {
 				'$set' => {
@@ -693,6 +898,8 @@ sub remove_game ($$) {
 			});
 		}
 	}
+	my $str_c = $self->get_collection('str');
+	$str_c->delete_many({game_id => $game->{_id}+0 });
 	my $games_c = $self->get_collection('games');
 	$games_c->delete_one({_id => $game->{_id}+0 });
 }
@@ -705,15 +912,261 @@ sub add_game ($$) {
 	my $games_c = $self->get_collection('games');
 	for my $t (0,1) {
 		for my $player (@{$game->{teams}[$t]{roster}}) {
+			my $method = $player->{position} eq 'G'
+				? sub { lc shift }
+				: sub { uc shift };
 			for my $stat (keys %{$player}) {
-				delete $player->{$stat} if ref $player->{$stat};
+				#				delete $player->{$stat} if ref $player->{$stat};
+				delete $player->{SHOT};
+				next if $STATS_FORCE_LC{$stat};
+				my $db_stat = $method->($stat);
+				$player->{$db_stat} = delete $player->{$stat};
 			}
 		}
 	}
-	$games_c->insert_one($game);
+	for my $field (keys %STATS_FORCE_NUMBER) {
+		$game->{$field} ||= 0;
+	}
+	numerify_structure($game);
+	my $shifts = delete $game->{shifts};
+	insert('shifts', @{$shifts});
+	ensure_shift_indices();
+	insert($games_c, $game);
 	$game->{_id};
 }
 
+sub get_catalog_condition ($$$) {
+
+	my $self    = shift;
+	my $catalog = shift;
+	my $name    = shift;
+
+	my $catalog_c = $self->get_collection($catalog);
+	my $item      = $catalog_c->find_one({name => $name})->{_id};
+
+	{ $catalog => $item };
+}
+
+sub get_item_collection ($$) {
+
+	my $self       = shift;
+	my $collection = shift;
+
+	$collection
+		? (ref $collection ? $collection : $self->get_collection($collection))
+		: $self->get_collection('games');
+
+}
+
+sub get_item_condition ($$) {
+
+	my $self       = shift;
+	my $condition  = shift;
+
+	my $_condition = {};
+	return $condition if delete $condition->{verbatim};
+	if ($condition) {
+		$_condition = {
+			map {
+				$self->get_catalog_condition($_, $condition->{$_})
+			} keys %{$condition}
+		};
+	}
+	$_condition;
+}
+
+sub find_items_with_games ($) {
+
+	my $self = shift;
+	my $opts = shift;
+
+	my $collection = $self->get_item_collection($opts->{collection});
+	my $condition  = $self->get_item_condition( $opts->{condition} );
+	my $id_field   = $opts->{id_field}   || '_id';
+	my $ids        = $opts->{game_ids}   || [];
+	my @return_fields = $opts->{return_fields}
+		? @{$opts->{return_fields}}
+		: ();
+
+	$condition->{$id_field} = { '$in' => $ids } if @{$ids};
+	my %return_fields = map { $_ => 1 } @return_fields;
+	my $method = $opts->{count} ? 'count_documents' : 'find';
+
+	my $iterator = $collection->$method(
+		$condition,
+	);
+	$iterator->fields(\%return_fields) if ! $opts->{count} && @return_fields;
+	$iterator;
+
+}
+
+sub find_player ($$$$$) {
+
+	my $self   = shift;
+	my $name   = shift;
+	my $team   = shift;
+	my $season = shift;
+	my $stage  = shift;
+	
+	return $EMPTY_NET_ID if $name eq 'EMPTY NET';
+
+	my $initial = ''; my $last_name = '';
+	if ($name =~ /^(\S.*?)\..*\s(\S.*)$/) {
+		$initial = $1;
+		$last_name = $2;
+	}
+	elsif ($name =~ /^(\S+)\s+(\S+)$/) {
+		$initial = substr($1, 0, 1);
+		$last_name = $2;
+	}
+	else {
+		$last_name = $name;
+	}
+	my $players_c = $self->get_collection('players');
+	my @players;
+	#       verbose "resolving $name / $initial.*$last_name";
+	@players = $players_c->find({name => $name})                         ->all();
+	@players = $players_c->find({name => qr/^$initial.*\b$last_name\b/i})->all() unless @players;
+	@players = $players_c->find({name => qr/\b$last_name\b/i})           ->all() unless @players;
+	@players = grep {$_->{active}} @players if @players > 1;
+	return $players[0]->{_id} if (@players == 1);
+	@players = grep {$_->{team} eq $team } @players if @players > 1;
+	return $players[0]->{_id} if (@players == 1);
+	$stage = $stage == $REGULAR ? 'regular' : 'playoff';
+	my @candidates = ();
+	for my $player (@players) {
+		push(@candidates, (grep {
+			$_->{start} ||= 0;
+			$_->{team} eq $team && $_->{start} == ($season || 0)
+		} @{$player->{career}[0]}, @{$player->{career}[1]}) ? $player : ());
+	}
+	return $candidates[0]->{_id} if @candidates == 1;
+	#       return $players[0]->{_id} if @players == 1;
+	if (@candidates > 1) {
+		dumper [ $name, $team, $season, $stage, ];
+		print Dumper [ map {$_->{_id}} @candidates ];
+		if ($ENV{HOCKEYDB_TEST}) {
+			warn "Couldn't resolve player";
+			return 0;
+		}
+		else {
+			confess "Couldn't resolve player";
+		}
+	}
+	elsif (@candidates == 0) {
+		dumper [ $name, $team, $season, $stage, ];
+		warn "Couldn't find player";
+		return 0;
+	}
+}
+
+
+sub bootstrap_teams ($) {
+
+	my $self = shift;
+	my $force = shift || 0;
+
+	my $teams_c = $self->get_collection('teams');
+	return if !$force && $teams_c->count_documents({});
+	$teams_c->delete_many({});
+	my @teams;
+	while (my ($_id, $values) = each %TEAMS) {
+		for my $key (keys %{$values}) {
+			next unless ref $values->{$key};
+			for my $name (@{$values->{$key}}) {
+				$name = uc $name;
+			}
+		}
+		$values->{defunct} ||= 0;
+		$values->{_id} = $_id;
+		push(@teams, $values);
+	}
+	debug "Bootstrapping teams...";
+	insert($teams_c, @teams);
+}
+
+sub bootstrap_divisions_conferences ($;$) {
+
+	my $self  = shift;
+	my $force = shift || 0;
+
+	my $divisions_c   = $self->get_collection('divisions');
+	my $conferences_c = $self->get_collection('conferenc>es');
+	my $seasons_c     = $self->get_collection('seasons');
+
+	return if !$force && $seasons_c->count_documents({});
+	for ($divisions_c, $conferences_c, $seasons_c) {
+		$_->delete_many({}) if $force;
+	}
+	for my $s_span (keys %SEASONS) {
+		$s_span =~ /^(\d{4})_(\d{4})$/;
+		my $start  = $1;
+		my $finish = $2;
+		my $conferences = $SEASONS{$s_span};
+		my @c_ids;
+		for my $conference (keys %{$conferences}) {
+			my $divisions = $conferences->{$conference};
+			my @d_ids;
+			for my $division (keys %{$divisions}) {
+				my $teams = $divisions->{$division};
+				my $d_c_id = $divisions_c->insert_one({
+					name => $division,
+					teams => $teams,
+				});
+				push(@d_ids, $d_c_id->{inserted_id});
+			}
+			my $c_c_id = $conferences_c->insert_one({
+				name => $conference,
+				divisions => [ @d_ids ],
+			});
+			push(@c_ids, $c_c_id->{inserted_id});
+		}
+		for my $year ($start..$finish) {
+			$seasons_c->insert_one(
+				{
+					year => $year+0,
+					conferences => [ @c_ids ],
+				},
+			);
+		}
+	}
+}
+
+sub insert_injuries ($$) {
+
+	my $self = shift;
+	my $injuries = shift;
+
+	my $i = 1;
+	my $injuries_c = $self->get_collection('injuries');
+	for my $injury (@{$injuries}) {
+		my $player = $self->find_player(
+			$injury->{player_name}, $injury->{team},
+			$CURRENT_SEASON, $CURRENT_STAGE,
+		) || (
+			$ENV{HOCKEYDB_TEST}
+			? $i++
+			: die "Couldn't find $injury->{player_name} on $injury->{team}"
+		);
+		my $player_injury = $injuries_c->find_one({
+			_id  => $player + 0,
+			team => $injury->{team},
+		});
+		if ($player_injury) {
+			update(
+				0, $injuries_c, { _id => $player+0 },
+				{ '$set' => { end => time }},
+			);
+		}
+		else {
+			$injury->{_id} = $player;
+			$injury->{begin} = time;
+			$injury->{end}   = time;
+			insert($injuries_c, $injury);
+		}
+	}
+}
+
 1;
 
 =head1 AUTHOR
